# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.0

import winrt.system
from winrt._winrt_windows_applicationmodel_payments_provider import (
    PaymentAppCanMakePaymentTriggerDetails,
    PaymentAppManager,
    PaymentTransaction,
    PaymentTransactionAcceptResult,
)

__all__ = [
    "PaymentAppCanMakePaymentTriggerDetails",
    "PaymentAppManager",
    "PaymentTransaction",
    "PaymentTransactionAcceptResult",
]

